
//	File:			InputHandler.java
//	Author:		Tarek Hassan
//	Date:			04 July 2001
//
//	Modified by:  Paul Marlow - January 2002

import javax.swing.*;
import java.awt.event.*;
import javax.swing.event.*;
import java.util.*;
import java.awt.*;

// takes mouse and keyboard input from user and puts appropriate commands in the commands queue
class InputHandler implements MouseInputListener, KeyListener
{
    public Vector m_commands;//= new Vector(5);
    private PlayerCanvas p_canvas;
    private Point user;
    private Point mousePosition;
    public double maxpower = 100; // server defaults assumed (max=100, min=-30)
    public double minpower = -30;
    private double powerScale = 0.5;   // powerScale * distance of mouse click from player = power
    private double power;
    public boolean running = false;

  public InputHandler(PlayerCanvas p)
  {
    p_canvas = p;
    m_commands = new Vector(5);
    user = p_canvas.getUserCoordinates();
  }

    
  // returns the distance of the point (x,y) from own player
  private double getDistance(int x, int y)
  {
    user = p_canvas.getUserCoordinates();
    double dist = Math.sqrt(((x - user.x)*(x - user.x))+((y - user.y)*(y - user.y)));
    //System.out.println("Distance = " + dist);
    return dist;
  }

  // returns the angle of the point (x,y) from straight ahead of own player
  private double getAngle(int x, int y)
  {
    user = p_canvas.getUserCoordinates();
    if (y != user.y) {    // to avoid divide by zero
      //System.out.println(user);
      //System.out.println("click @("+x+","+y+")");
      double ang = Math.toDegrees(Math.atan((double)(x-user.x)/(user.y-y)));
      //System.out.println("Angle = " + ang);
      if ((x > user.x)&&(ang < 0)){
        ang += 180;
      }
      if ((x < user.x)&&(ang > 0)){
        ang -= 180;
      }
      if ((x == user.x)&&(y > user.y)){
        ang = 180;
      }
      return ang;
    } else if (x > user.x) {
      return 90;
    } else return -90;
  }

  //===========================================================================
  // The implementation of MouseInputListener interface

  // left click = dash with power relative to distance of mouse click from own player
  // left hold = run (repeatedly dash every cycle at the same power)
  // right click = turn with moment equal to the angle from straight ahead of own player
  public void mousePressed(MouseEvent e)
  {
    mousePosition = new Point(e.getX(), e.getY());
    if (!m_commands.isEmpty()){
      m_commands.removeAllElements();
    }
    // check if left mouse button
    if (e.getModifiers()== 16){
      running = true;
      if (mousePosition.y <= user.y){
        power = powerScale * getDistance(mousePosition.x, mousePosition.y);
        if (power > maxpower){
          power = maxpower;
        }
      } else {
        power = -powerScale * getDistance(mousePosition.x, mousePosition.y);
        if (power < minpower){
          power = minpower;
        }
      }
      Command dash = new Command("dash", power);
      m_commands.addElement(dash);
    } else {
      double test = Math.toDegrees(Math.atan(5));
      double moment = getAngle(mousePosition.x, mousePosition.y);
      Command turn = new Command("turn", moment);
      m_commands.addElement(turn);
      running = false;
    }
  }

  public void mouseReleased(MouseEvent e)
  {
    mousePosition = new Point(e.getX(), e.getY());
    running = false;
  }

  public void mouseClicked(MouseEvent e)
  {
  }

  public void mouseEntered(MouseEvent e)
  {
  }

  public void mouseExited(MouseEvent e)
  {
  }

  public void mouseMoved(MouseEvent e)
  {
    mousePosition = new Point(e.getX(), e.getY());
  }

  public void mouseDragged(MouseEvent e)
  {
    mousePosition = new Point(e.getX(), e.getY());
  }

  //===========================================================================
  // The implementation of KeyListener interface

  /**
   * Invoked when a key has been typed.
   * This event occurs when a key press is followed by a key release.
   */
  public void keyTyped(KeyEvent e)
  {
  }

  /**
   * Invoked when a key has been pressed.
   */
  public void keyPressed(KeyEvent e)
  {
    if (!m_commands.isEmpty()){
      m_commands.removeAllElements();
    }

    // space for kick.  direction and power depending on relative mouse position
    //System.out.println("Key Pressed " + e.getKeyChar());
    if (e.getKeyChar() == ' '){
      double power = powerScale * getDistance(mousePosition.x, mousePosition.y);
      if (power > maxpower){
        power = maxpower;
      }
      double angle = getAngle(mousePosition.x, mousePosition.y);
      Command kick = new Command("kick", power, angle);
      m_commands.addElement(kick);    
      running = false;
    }
  }
    
  /**
   * Invoked when a key has been released.
   */
  public void keyReleased(KeyEvent e)
  {
  }
}
