//
//	File:			VisualInfo.java
//	Author:		Krzysztof Langner
//	Date:			1997/04/28

//  Modified by:  Paul Marlow, Amir Ghavam, Yoga Selvaraj
//  Course:       Software Agents
//  Date Due:     November 30, 2000 

//  Modified by:  Tarek Hassan
//  Date:         15 June 2001

//  Modified by:  Paul Marlow - January 2002

import java.util.*;

class VisualInfo
{
  private int m_time;
  public Vector m_objects;
  public StringTokenizer m_tokenizer;
  public String m_message;
  public Vector visuals;

  // Inserts visual objects into a list, sorted ascending by distance - used
  // for drawing the canvas (i.e. most flags are not entered into the list)
  private void insertVisuals(ObjectInfo obj)
  {
    if (visuals.isEmpty()) {
      visuals.add(obj);
    } else {
      // Find where to insert the object
      int i = 0;
      for (i = 0; i < visuals.size(); i++) {
        if (obj.m_distance <= ((ObjectInfo) visuals.elementAt(i)).m_distance) {
          break;
        }
      }

      // Insert the object
      visuals.insertElementAt(obj, i);
    }
  } // private insertVisuals(ObjectInfo obj)

  // Used after parsing VisualInfo.  It fills the visuals Vector with those
  // objects to be drawn on the main canvas - ball, players and goal flags.
  private void addVisuals()
  {
    ObjectInfo obj;
    for (int i = 0; i < m_objects.size(); i++)
    {
      obj = (ObjectInfo)(m_objects.elementAt(i));

      // Only insert if the object is a ball, player, or a goal post
      //System.out.println(obj.m_type);
      if (/*(obj.m_type.compareToIgnoreCase("ball") == 0) || (obj.m_type.compareToIgnoreCase("player") == 0) ||
          (obj.m_type.compareTo("flag g l t") == 0) || (obj.m_type.compareTo("flag g l b") == 0) ||
          (obj.m_type.compareTo("flag g r t") == 0) || (obj.m_type.compareTo("flag g r b") == 0) */
          !(obj.m_type.compareTo("goal 1") == 0) && !(obj.m_type.compareTo("goal r") == 0))
      {
        insertVisuals(obj);
      }
    }
  } // private addVisuals(ObjectInfo obj)

  //---------------------------------------------------------------------------
  // Constructor for 'see' information
  public VisualInfo(String info)
  {
    info.trim();
    m_message = info;
    m_tokenizer = new StringTokenizer(info,"() ", true);
    m_objects = new Vector(100);
    visuals = new Vector(30);
  }

  //---------------------------------------------------------------------------
  // This function parses visual information from the server
  public void parse()
  {
    String token = null;
    ObjectInfo objInfo;
    //visuals.clear();

    try {
      m_tokenizer.nextToken();	// '('
      // Don't parse information if it's not 'see' information
      if( m_tokenizer.nextToken().compareTo("see") != 0 )
        return;

      m_tokenizer.nextToken();	// ' '
      m_time = Integer.parseInt( m_tokenizer.nextToken() ); // TIME
      m_tokenizer.nextToken();	// ' '
      token = m_tokenizer.nextToken();

      while(token.compareTo("(") == 0)// '('
      {
        // Create soccer object for reference - can be player, ball, flag or line
        objInfo = createNewObject();
        m_objects.addElement(objInfo);

        token = m_tokenizer.nextToken();
        if( token.compareTo(")") == 0 )
        {
          token = m_tokenizer.nextToken();
          if(token.compareTo(" ") == 0) token = m_tokenizer.nextToken();
          continue;
        }

        token = m_tokenizer.nextToken();
        // Get object's approximate distance from agent - actual distance
        // becomes more difficult to determine as the distance increases
        objInfo.m_distance = Float.valueOf(token).floatValue();
        token = m_tokenizer.nextToken();

        if( token.compareTo(")") == 0 )
        {
          token = m_tokenizer.nextToken();
          if(token.compareTo(" ") == 0) token = m_tokenizer.nextToken();
          continue;
        }

        // Get object direction relative to the agent
        objInfo.m_direction = Float.valueOf(m_tokenizer.nextToken()).floatValue();
        token = m_tokenizer.nextToken();

        if( token.compareTo(")") == 0 )
        {
          token = m_tokenizer.nextToken();
          if(token.compareTo(" ") == 0) token = m_tokenizer.nextToken();
          continue;
        }

        // Get the distance change since last 'see' message
        objInfo.m_distChange = Float.valueOf(m_tokenizer.nextToken()).floatValue();
        token = m_tokenizer.nextToken();

        if( token.compareTo(")") == 0 )
        {
          token = m_tokenizer.nextToken();
          if(token.compareTo(" ") == 0) token = m_tokenizer.nextToken();
          continue;
        }

        // Get direction change since last 'see' message
        objInfo.m_dirChange = Float.valueOf(m_tokenizer.nextToken()).floatValue();
        token = m_tokenizer.nextToken();

        if( token.compareTo(")") == 0 )
        {
          token = m_tokenizer.nextToken();
          if(token.compareTo(" ") == 0) token = m_tokenizer.nextToken();
          continue;
        }

        // Player's bodyDir parameter
        ((PlayerInfo)(objInfo)).m_bodyDir = Float.valueOf(m_tokenizer.nextToken()).floatValue();
        token = m_tokenizer.nextToken();

        if( token.compareTo(")") == 0 )
        {
          token = m_tokenizer.nextToken();
          if(token.compareTo(" ") == 0) token = m_tokenizer.nextToken();
          continue;
        }

        // Player's headDir parameter
        ((PlayerInfo)(objInfo)).m_headDir = Float.valueOf(m_tokenizer.nextToken()).floatValue();

        token = m_tokenizer.nextToken();
        token = m_tokenizer.nextToken();
        if(token.compareTo(" ") == 0) token = m_tokenizer.nextToken();
      }

      // Check to see if the object is a visual to be drawn, and if so insert it
      // into the visuals Vector
      addVisuals();
    }
    catch(Exception e)
    {
      System.err.println("Error parsing see information!");
      System.err.println(token);
      while( m_tokenizer.hasMoreTokens() )
      {
        token = m_tokenizer.nextToken();
        System.out.print(token);//m_tokenizer.nextToken());
      }
      System.out.println("");
    }
  }

  //===========================================================================
  // Private implementations

  //---------------------------------------------------------------------------
  // This function creates new object based on the see message sent from the server
  private ObjectInfo createNewObject()
  {
    String token = null;
    ObjectInfo objInfo = null;

    m_tokenizer.nextToken();			// '('
    token = m_tokenizer.nextToken();
    if(token.compareTo("player") == 0)  // player object
    {
      String team = new String();
      int uniformNumber = 0;
      token = m_tokenizer.nextToken();	// ' '
      if(token.compareTo(" ") == 0)
      {
        team = m_tokenizer.nextToken();	// teamname
        token = m_tokenizer.nextToken();// ' '
        if(token.compareTo(" ") == 0)
        {
          // uniform number
          uniformNumber = Integer.parseInt(m_tokenizer.nextToken());
        }
      }
      objInfo = new PlayerInfo(team, uniformNumber);
    }
    else if(token.compareTo("goal") == 0) // goal object
    {
      token = m_tokenizer.nextToken();	// space
      if(token.compareTo(" ") == 0) token = m_tokenizer.nextToken();	// side
      objInfo = new GoalInfo(token.charAt(0));
    }
    else if(token.compareTo("ball") == 0) // ball object
    {
      objInfo = new BallInfo();
    }
    else if(token.compareTo("flag") == 0) // flag object
    {
      char type = ' ';
      char pos1 = ' ';
      char pos2 = ' ';
      int num = 0;
      boolean out = true;

      token = m_tokenizer.nextToken();	// space
      token = m_tokenizer.nextToken();	// p or g or [l|c|r] or [b|t]
      if(( token.compareTo("p") == 0 )||( token.compareTo("g") == 0 ))
      {
        type = token.charAt(0);
        token = m_tokenizer.nextToken();	// space
        token = m_tokenizer.nextToken();	// [l|r]
        pos1 = token.charAt(0);
        token = m_tokenizer.nextToken();	// space
        token = m_tokenizer.nextToken();	// [t|c|b]
        pos2 = token.charAt(0);
        out = false;
      }
      else if(( token.compareTo("l") == 0 )||( token.compareTo("r") == 0 ))
      {
        pos1 = token.charAt(0);
        token = m_tokenizer.nextToken();	// space
        token = m_tokenizer.nextToken();	// [t|b] or 0
        pos2 = token.charAt(0);

        if (pos2 == '0')
        {
          num=0;
          pos2=' ';
          out=true;
        }
        else
        {
          token = m_tokenizer.nextToken(); 	// space or )
          if ( token.compareTo(")") == 0 ) out=false;
          else
          {
            num = Integer.parseInt( m_tokenizer.nextToken() );
            out=true;
          }
        }
      }
      else if( token.compareTo("c") == 0 )
      {
        pos1 = token.charAt(0);
        token = m_tokenizer.nextToken();	// ) or space
        if ( token.compareTo(")") != 0)
        {
          token = m_tokenizer.nextToken();	// [t|b]
          pos2 = token.charAt(0);
        }
        out=false;
      }
      else if(( token.compareTo("b") == 0 )||( token.compareTo("t") == 0 ))
      {
        pos1 = token.charAt(0);
        token = m_tokenizer.nextToken();	// space
        token = m_tokenizer.nextToken();	// [r|l] or 0
        pos2 = token.charAt(0);

        if ( pos2 == '0')
        {
          pos2=' ';
          num=0;
        }
        else
        {
          token = m_tokenizer.nextToken();			// space
          num = Integer.parseInt(m_tokenizer.nextToken());	// number
          token = m_tokenizer.nextToken();			// )
        }
        out=true;
      }

      String flagType = "flag";
      if (type != ' ') flagType = flagType + " " + type;
      if (pos1 != ' ') flagType = flagType + " " + pos1;
      if (pos2 != ' ') flagType = flagType + " " + pos2;
      // Implementing flags like this, allows one to specifically find a
      // particular flag (i.e. "flag c", or "flag p l t")
      objInfo = new FlagInfo(flagType, type, pos1, pos2, num, out);
    }
    else if(token.compareTo("line") == 0) // line object
    {
      token = m_tokenizer.nextToken();	// space
      token = m_tokenizer.nextToken();	// [l|r|t|b]
      objInfo = new LineInfo(token.charAt(0));
    }
    else if(token.compareTo("Player") == 0)
    {
      objInfo = new PlayerInfo();
    }
    else if(token.compareTo("Goal") == 0)
    {
      objInfo = new GoalInfo();
    }
    else if(token.compareTo("Ball") == 0)
    {
      objInfo = new BallInfo();
    }
    else if(token.compareTo("Flag") == 0)
    {
      objInfo = new FlagInfo();
    }
    else if(token.compareTo("Line") == 0)
    {
      objInfo = new LineInfo();
    }

    while (token.compareTo(")") != 0)
    {
      token = m_tokenizer.nextToken();
    }

    return objInfo;
  }
}

